@@ -98,9 +98,9 @@ func (h *Head) initTime(t int64) {
 	h.maxTime.CompareAndSwap(math.MinInt64, t)
 }
 
-func (a *initAppender) GetRef(lset labels.Labels) (storage.SeriesRef, labels.Labels) {
+func (a *initAppender) GetRef(lset labels.Labels, hash uint64) (storage.SeriesRef, labels.Labels) {
 	if g, ok := a.app.(storage.GetRef); ok {
-		return g.GetRef(lset)
+		return g.GetRef(lset, hash)
 	}
 	return 0, nil
 }
@@ -440,7 +440,7 @@ func (s *memSeries) appendableHistogram(t int64, h *histogram.Histogram) error {
 }
 
 // AppendExemplar for headAppender assumes the series ref already exists, and so it doesn't
-// use getOrCreate or make any of the lset sanity checks that Append does.
+// use getOrCreate or make any of the lset validity checks that Append does.
 func (a *headAppender) AppendExemplar(ref storage.SeriesRef, lset labels.Labels, e exemplar.Exemplar) (storage.SeriesRef, error) {
 	// Check if exemplar storage is enabled.
 	if !a.head.opts.EnableExemplarStorage || a.head.opts.MaxExemplars.Load() <= 0 {
@@ -647,8 +647,8 @@ func checkHistogramBuckets(buckets []int64) (uint64, error) {
 
 var _ storage.GetRef = &headAppender{}
 
-func (a *headAppender) GetRef(lset labels.Labels) (storage.SeriesRef, labels.Labels) {
-	s := a.head.series.getByHash(lset.Hash(), lset)
+func (a *headAppender) GetRef(lset labels.Labels, hash uint64) (storage.SeriesRef, labels.Labels) {
+	s := a.head.series.getByHash(hash, lset)
 	if s == nil {
 		return 0, nil
 	}
